import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import "../components"
import "./view"

import Artix.Handlers
import Artix.Models
import Artix.Core
import Artix.Gui

/**
 * @brief Форма открытого документа
 */
BasicView {
    id: root

    // Фильтр и ресивер для работы с клавиатурой
    property BasicTextField eventReceiver: inputField
    property FormEventFilter eventFilter: openDocumentEventFilter

    FormEventFilter {
        id: openDocumentEventFilter
        onKeyboardData: (data, rawData, source) => {
                            inputField.text = ""
                            handler.input(data, rawData, source)
                        }
        onOk: {
            if (inputField.text.length === 0)
                return
            let text = inputField.text
            inputField.text = ""
            handler.input(text, text, InputSource.Keyboard)
        }
        onCancel: {
            inputField.text = ""
            handler.escCommand()
        }
    }

    contentItem: RowLayout {
        spacing: 24
        // Левая часть - список позиций
        GroupBox {
            Style.class: "positionsListBackground"
            Layout.fillHeight: true
            Layout.fillWidth: true
            ColumnLayout {
                anchors.fill: parent
                spacing: 0
                // Информационные оповещения
                SingleNotification {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 40
                    filter: Notification.Info
                    interactive: false
                }
                // Шапка списка позиций
                PositionsListHeader {
                    id: positionListHeader
                    Layout.fillWidth: true
                    Layout.preferredHeight: 57
                    positionsCount: positions.model.positionsCount
                }
                // Список позиций
                PositionsList {
                    id: positions
                    clip: true
                    withModifiers: true
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    scrollToEndOnItemAdded: true
                    onStorno: (position) => { handler.stornoPosition(position) }
                    onChangeQuantity: (position, value) => { handler.changeQuantity(position, value) }
                    onConsultantDelete: (position) => { handler.consultantDelete(position) }
                }
                // Дополнительная навигация
                ListNavigation {
                    target: positions
                    Layout.fillWidth: true
                    Layout.preferredHeight: 116
                    visible: Style.parameter("capacitiveTouchscreen") === "false"
                }
            }
        }
        // Правая часть - элементы управления
        Item {
            Layout.preferredWidth: Math.floor((screenWidth * 0.19 + 374) / 8) * 8
            Layout.fillHeight: true
            // Верхняя панель кассы
            TopPanel {
                id: topPanel
                anchors { left: parent.left; top: parent.top; right: parent.right }
            }
            ColumnLayout {
                anchors { left: parent.left; bottom: parent.bottom; right: parent.right; top: topPanel.bottom }
                anchors { rightMargin: 24; bottomMargin: 24; topMargin: errorNotification.visible ? 0 : 24 }
                spacing: 0
                // Критические оповещения
                SingleNotification {
                    id: errorNotification
                    Layout.fillWidth: true
                    Layout.preferredHeight: 40
                    filter: Notification.Error | Notification.Mode
                    onClicked: topPanel.open()
                }
                Item { Layout.preferredHeight: errorNotification.visible ? 4 : 0 }
                // Сумма чека/заголовок панели действий
                Item {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 48
                    BasicLabel {
                        id: title
                        anchors.left: parent.left
                        anchors.bottom: parent.bottom
                        Style.class: "documentOpenTotalTitle"
                        property var defaultTr:
                            Utils.makeTr("documentOpenTitleTotal", "Сумма")
                        Tr { id: titleTr; tr: titleTr.defaultTr }
                    }
                    BasicLabel {
                        id: totalValue
                        anchors.right: parent.right
                        anchors.verticalCenter: parent.verticalCenter
                        Style.class: "documentOpenTotalValue"
                        text: Utils.formatMoneyItem(Session.document.sumb)
                        horizontalAlignment: Text.AlignRight
                    }
                }
                Item { Layout.preferredHeight: errorNotification.visible ? 8 : 24 }
                // Поле ввода
                BasicTextField {
                    id: inputField
                    property var defaultTr:
                        Utils.makeTr("documentOpenInputBarcodePlaceholder", "Введите код товара")

                    Layout.fillWidth: true
                    Layout.preferredHeight: 64
                    Style.class: "documentOpenInput"
                    padding: 12
                    verticalAlignment: Text.AlignVCenter
                    horizontalAlignment: Text.AlignRight
                    hideCursor: true
                    fixedCursorPosition: true
                    BasicLabel {
                        Style.class: "documentOpenInputPlaceholder"
                        anchors.left: parent.left
                        anchors.bottom: parent.bottom
                        anchors.margins: 12
                        visible: inputField.text.length === 0
                        Tr { id: inputFieldPlaceholderTr; tr: inputField.defaultTr }
                    }
                    onClicked: {
                        if (actionPanelStack.depth === 1)
                            actionPanelStack.push("./view/actionPanelExtensions/TmcActionPanel.qml")
                    }
                }
                Item { Layout.preferredHeight: errorNotification.visible ? 20 : 24 }
                // Панель действий
                StackView {
                    id: actionPanelStack
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    initialItem: ActionPanel {
                        id: actionPanel
                        context: Context.DocumentOpen
                        onClicked: (actionCode) => { handler.processAction(actionCode) }
                    }
                    popEnter: null
                    popExit: null
                    pushEnter: null
                    pushExit: null
                    onCurrentItemChanged: {
                        if (actionPanelStack.depth === 1) {
                            root.eventFilter = openDocumentEventFilter
                            inputFieldPlaceholderTr.tr = inputField.defaultTr
                            titleTr.tr = title.defaultTr
                            totalValue.opacity = 1.0
                            return
                        }

                        if (currentItem.eventFilter)
                            root.eventFilter = currentItem.eventFilter
                        if (currentItem.inputFieldMask) {
                            inputField.validator = currentItem.inputFieldMask
                            inputField.text = ""
                        }
                        currentItem.text = Qt.binding(() => { return inputField.text })
                        if (currentItem.placeholder !== undefined)
                            inputFieldPlaceholderTr.tr = currentItem.placeholder
                        if (currentItem.title !== undefined) {
                            titleTr.tr = currentItem.title
                            totalValue.opacity = 0.0
                        }
                        currentItem.back?.connect(() => {
                                                      inputField.text = ""
                                                      actionPanelStack.pop()
                                                  })
                        currentItem.commit?.connect((action, args) => {
                                                        actionPanelStack.pop()
                                                        inputField.text = ""
                                                        handler.emitAction(action, args)
                                                   })
                        currentItem.command?.connect((command, args) => {
                                                         actionPanelStack.pop()
                                                         inputField.text = ""
                                                         handler.emitAction(command, args)
                                                     })
                        currentItem.input?.connect((data) => {
                                                       actionPanelStack.pop()
                                                       inputField.text = ""
                                                       handler.input(data, data, InputSource.Keyboard)
                                                   })
                    }
                }
            }
        }
    }
    // Нижняя панель кассы
    footer: BottomPanel {}

    // Обработчик событий
    DocumentOpenHandler {
        id: handler
        onPageTo: (page) => { actionPanel.pushPage(page) }
        onPageBack: actionPanel.popPage()
        onPanelExtension: (viewName, action, position) => {
                              actionPanelStack.push("./view/actionPanelExtensions/" + viewName + ".qml")
                              actionPanelStack.currentItem.action = action
                              inputField.text = actionPanelStack.currentItem.getDefaultInputValue()
                              if (actionPanelStack.currentItem.hasOwnProperty("position"))
                                actionPanelStack.currentItem.position = position
                          }
        onDocumentChanged: positionListHeader.updateDocumentLabel(getDocumentInfo())
        Component.onCompleted: positionListHeader.updateDocumentLabel(getDocumentInfo())
    }
}
