import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import Qt.labs.qmlmodels

import "../components"
import "./view"

import Artix.Handlers
import Artix.Core
import Artix.Gui
import Artix.Models

/**
 * @brief Форма отображения информации о документе
 */
BasicView {
    id: root
    readonly property bool fullScreen: true
    implicitWidth: Screen.desktopAvailableWidth
    implicitHeight: Screen.desktopAvailableHeight
    property var document: Utils.variantToObjectPointer(params.document)

    // Фильтр для работы с клавиатурой
    property DialogEventFilter eventFilter: DialogEventFilter {
        onOk: handler.sendAnswer(true)
        onCancel: handler.sendAnswer(false)
    }

    contentItem: RowLayout {
        // Левая часть
        Rectangle {
            Layout.fillWidth: true
            Layout.fillHeight: true
            color: Style.parameter("color.modalBackground")
            ColumnLayout {
                anchors.fill: parent
                spacing: 16
                Item {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 100
                    Layout.topMargin: 24
                    Layout.leftMargin: 24
                    Layout.rightMargin: 24
                    ColumnLayout {
                        anchors.fill: parent
                        spacing: 0
                        BasicLabel {
                            Style.class: "documentViewerTitle"
                            Tr { code: "documentViewerTitle"; defaultMessage: "Просмотр" }
                        }
                        Item { Layout.preferredHeight: 12 }
                        RowLayout {
                            spacing: 16
                            BasicLabel {
                                Style.class: "documentViewerInfo"
                                Tr {
                                    code: "documentViewerDocumentNum"; defaultMessage: "Продажа №%1";
                                    args: [document?.num]
                                }
                            }
                            BasicLabel {
                                Style.class: "documentViewerInfo"
                                Tr {
                                    code: "documentViewerDocumentShiftNum"; defaultMessage: "Смена %1"
                                    args: [document?.shift]
                                }
                            }
                            BasicLabel {
                                Style.class: "documentViewerInfo"
                                Tr {
                                    code: "documentViewerDocumentDateTimeEnd"; defaultMessage: "Закрыт %1"
                                    args: [document?.dateTimeEnd.toLocaleString(Qt.locale("en_US"), "dd.MM.yyyy hh:mm")]
                                }
                            }
                        }
                        Item { Layout.preferredHeight: 8 }
                        BasicLabel {
                            Style.class: "documentViewerInfo"
                            Tr {
                                code: "documentViewerDocumentUserName"; defaultMessage: "Кассир %1"
                                args: [document?.user.name]
                            }
                        }
                    }
                }
                Line {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 2
                    strokeWidth: 2
                    color: Style.parameter("color.grayDark")
                }
                ListView {
                    id: positions
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    Layout.leftMargin: 24
                    Layout.rightMargin: 20
                    model: DocumentModel { document: params.document }
                    clip: true
                    delegate: GroupBox {
                        Style.class: "documentViewerPositionDelegate"
                        width: positions.width
                        height: 44
                        interactive: false
                        contentItem: RowLayout {
                            visible: type === DocumentPositionType.Goods
                            anchors.fill: parent
                            spacing: 12
                            BasicLabel {
                                Style.class: "documentViewerPositionDelegate"
                                Layout.fillHeight: true
                                Layout.preferredWidth: 32
                                text: object.posnum
                                verticalAlignment: Label.AlignVCenter
                            }
                            BasicLabel {
                                Style.class: "documentViewerPositionDelegate"
                                Layout.fillWidth: true
                                Layout.fillHeight: true
                                text: object.name
                                verticalAlignment: Label.AlignVCenter

                                elide: Text.ElideRight
                                wrapMode: Text.WordWrap
                                maximumLineCount: 2
                            }
                            BasicLabel {
                                Style.class: "documentViewerPositionDelegate"
                                Layout.fillHeight: true
                                Layout.preferredWidth: 60
                                text: Utils.weightStr(object.bquant)
                                verticalAlignment: Label.AlignVCenter
                            }
                            BasicLabel {
                                Style.class: "documentViewerPositionDelegate"
                                Layout.fillHeight: true
                                Layout.preferredWidth: 110
                                Layout.rightMargin: 4   // Нужно из-за скроллбара
                                text: Utils.formatMoneyItem(object.sumb)
                                horizontalAlignment: Label.AlignRight
                                verticalAlignment: Label.AlignVCenter
                            }
                        }
                    }
                    ScrollBar.vertical: ScrollBar {
                        id: scrollBar
                        Timer {
                            id: scrollTimer
                            interval: 2500
                            onTriggered: parent.active = false
                        }
                        onActiveChanged: scrollTimer.start()
                    }
                }
                GroupBox {
                    Style.class: "documentViewerPayments"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 136
                    Layout.leftMargin: 24
                    Layout.rightMargin: 24
                    Layout.bottomMargin: 24
                    padding: 24
                    interactive: false
                    contentItem: ColumnLayout {
                        spacing: 16
                        Item {
                            Layout.fillWidth: true
                            Layout.preferredHeight: 24
                            BasicLabel {
                                Style.class: "documentViewerPaymentSum"
                                anchors.left: parent.left
                                anchors.verticalCenter: parent.verticalCenter
                                Tr { code: "documentViewerPaymentSum"; defaultMessage: "Сумма чека" }
                            }
                            BasicLabel {
                                Style.class: "documentViewerPaymentSum"
                                anchors.right: parent.right
                                anchors.verticalCenter: parent.verticalCenter
                                text: Utils.formatMoneyItem(document?.sumb)
                            }
                        }
                        PaymentList {
                            document: params.document
                            Layout.fillWidth: true
                            Layout.fillHeight: true
                        }
                    }
                }
            }
        }
        // Правая часть
        Item {
            Layout.fillHeight: true
            Layout.preferredWidth: 304
            TopPanel {
                anchors { left: parent.left; top: parent.top; right: parent.right }
                anchors.leftMargin: 24
                hideUserName: true
            }
            ColumnLayout {
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 24 }
                spacing: 8
                BasicButton {
                    Style.class: "documentViewerCancel"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    Tr { code: "documentViewerFormCancelButton"; defaultMessage: "Назад" }
                    onClicked: handler.sendAnswer(false)
                }
                BasicButton {
                    Style.class: "documentViewerOk"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    iconLayout.alignment: Qt.AlignTop | Qt.AlignRight
                    Tr { code: "documentViewerFormOkButton"; defaultMessage: "Выбрать" }
                    icon: Style.parameter("icon.actionPanel.arrow")
                    onClicked: handler.sendAnswer(true)
                }
            }
        }
    }

    footer: BottomPanel {}

    DialogHandler { id: handler }
}
