import QtQuick
import QtQuick.Window
import QtQuick.Controls
import QtQuick.VirtualKeyboard.Settings

import "../components"
import "./view"

import Artix.Core
import Artix.Gui

/**
 * @brief Главное окно программы
 */
ApplicationWindow {
    id: mainWindow
    visible: true
    width: screenWidth
    height: screenHeight
    readonly property var sidePanelRef: sidePanel

    Component.onCompleted: {
        let locales = Config.getStringList("Language:keyboardLayouts", "ru,en")
        let activeLocales = []
        for (let i = 0; i < locales.length; i++) {
            switch(locales[i]) {
                case "ru": activeLocales.push("ru_RU"); break;
                case "en": activeLocales.push("en_US"); break;
            }
        }
        VirtualKeyboardSettings.activeLocales = activeLocales
        requestActivate()
    }

    // Рабочая область
    Control {
        id: workArea
        anchors.fill: parent

        onContentItemChanged: {
            contentItem?.onEventFilterChanged?.connect(() => updateEventFilter())
            contentItem?.onEventReceiverChanged?.connect(() => updateEventFilter())
            updateEventFilter()
        }

        // Обновить фильтр формы и ресивер ввода для текущего элемента
        function updateEventFilter() {
            let eventFilter = contentItem?.eventFilter
            let eventReceiver = contentItem?.eventReceiver
            eventFilterConnector.update(eventFilter, eventReceiver)
        }
    }

    // Диалоги
    Popup {
        id: dialog
        padding: 0
        property real yShift: 24
        x: (mainWindow.width - width) / 2
        y: (mainWindow.height - height) / 2 - yShift
        modal: true
        closePolicy: Popup.NoAutoClose
        background: Item {}
        Overlay.modal: Rectangle { color: Style.parameter("color.modalShading") }
        // Стек используется для корректной работы при повторном отображении уже созданного элемента
        contentItem: StackView {
            id: dialogStack
            pushEnter: null
            pushExit: null
            popEnter: null
            popExit: null
            replaceEnter: null
            replaceExit: null
        }
    }

    // Боковая панель
    Drawer {
        id: sidePanel
        padding: 0
        modal: true
        clip: true
        dragMargin: 0.0
        edge: Qt.RightEdge
        background: Item {}
        Overlay.modal: Rectangle { color: Style.parameter("color.modalShading") }
        property Item menuItem: null
        contentItem: StackView {
            id: sidePanelStack
            pushEnter: null
            pushExit: null
            popEnter: null
            popExit: null
            replaceEnter: null
            replaceExit: null
        }

        function createMenu() {
            closePolicy = Popup.CloseOnPressOutside
            sidePanelStack.clear()
            if (menuItem)
                menuItem.destroy()
            let comp = Qt.createComponent("view/SidePanel.qml")
            if (comp.status !== Component.Ready) {
                console.log(comp.errorString())
                return
            }
            menuItem = comp.createObject()
            if (menuItem == null) {
                console.log("Error creating side panel menu object")
                return
            }
            sidePanelStack.push(menuItem)
        }
    }

    // Коннектим сигналы из C++ движка
    Connections {
        id: connections
        target: engine

        // Показать представление контекста
        function onShowContextView(item) {
            workArea.contentItem = item
        }

        // Скрыть представление контекста
        function onCloseContextView() {
            workArea.contentItem = null
        }

        // Показать представление диалога
        function onShowDialog(item) {
            eventFilterConnector.update(item.eventFilter, item.eventReceiver)
            if (item.sideDialog) {
                sidePanelStack.push(item)
                sidePanel.closePolicy = Popup.NoAutoClose
                sidePanel.open()
            } else {
                dialogStack.implicitWidth = item.implicitWidth
                dialogStack.implicitHeight = item.implicitHeight
                dialogStack.push(item)
                if (item.fullScreen) {
                    dialog.yShift = 0
                } else {
                    dialog.yShift = 24
                }
                dialog.open()
            }
        }

        // Скрыть представление диалога
        function onCloseDialog(sideDialog) {
            if (sideDialog) {
                sidePanelStack.clear()
                sidePanel.close()
            } else {
                dialogStack.clear()
                dialog.close()
            }

            // После закрытия диалога, возврашаем ввод текущему элементу рабочей области
            workArea.updateEventFilter()
        }
    }

    EventFilterConnector {
        id: eventFilterConnector

        // Обновление фильтра и ресивера для текущей открытой формы
        function update(eventFilter, eventReceiver) {
            clear()
            if (eventFilter)
                filter = eventFilter
            if (eventReceiver)
                receiver = eventReceiver
        }
    }
}
