import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import Qt.labs.qmlmodels

import "../components"
import "./view"

import Artix.Core
import Artix.Gui
import Artix.Handlers
import Artix.Models

/**
 * @brief Форма подбора товаров из каталога
 * @param fullScreen диалог является полноэкранным
 */
BasicView {
    id: root

    implicitWidth: Screen.desktopAvailableWidth
    implicitHeight: Screen.desktopAvailableHeight
    padding: 24

    property BasicTextField eventReceiver: inputField
    property DialogEventFilter eventFilter: DialogEventFilter {
        id: eventFilter
        onOk: inputField.focus = false
        onCancel: {
            if (inputField.focus) {
                inputField.focus = false
                return
            }
            if (inputField.text.length > 0) {
                inputField.text = ""
                return
            }
            if (stack.length > 0) {
                pop()
                return
            }
            handler.sendAnswer()
        }
    }

    readonly property bool fullScreen: true

    property string rootGroup: params.rootGroup ///< корневой каталог
    property string currentName: ""             ///< имя текущего каталога
    property string headerText                  ///< текст "Каталог" в загаловке
    property list<var> stack: []                ///< стек каталогов

    Tr on headerText { code: "pickListFormCatalog"; defaultMessage: "Каталог" }
    onCurrentNameChanged: {
        headerModel.clear()
        headerModel.append({"name": headerText})
        for (let i = 0; i < stack.length; i++) {
            if (stack[i].name.length === 0 )
                continue
            headerModel.append({"name": stack[i].name})
        }
        if (currentName.length > 0)
            headerModel.append({"name": currentName})
    }

    // !!!!!!!!!! Делегат каталога !!!!!!!!!!
    component GroupDelegate: GroupBox {
        id: groupDelegate
        Style.class: "pickListDelegate"
        padding: 12
        rightPadding: 22
        topPadding: 22
        contentItem: ColumnLayout {
            spacing: 10
            Image {
                Layout.fillWidth: true
                Layout.preferredHeight: 120
                source: image
                fillMode: Image.PreserveAspectFit
            }
            BasicLabel {
                Style.class: "pickListDelegateName"
                Layout.fillWidth: true
                Layout.fillHeight: true
                text: name
                elide: Label.ElideRight
            }
        }
        background: Item {
            Repeater {
                model: 3
                delegate: Rectangle {
                    x: 5 * (2 - index)
                    y: 5 * index
                    width: parent.width - 10
                    height: parent.height - 10
                    color: groupDelegate.Style.background.color
                    border.color: groupDelegate.Style.background.border.color
                    border.width: groupDelegate.Style.background.border.width
                    radius: groupDelegate.Style.background.radius

                    Behavior on color { ColorAnimation { id: colorAnimation; duration: 100 } }

                    Rectangle {
                        anchors.fill: parent
                        opacity: 0.15 * (2 - index)
                        color: Style.parameter("color.shadow")
                        radius: groupDelegate.Style.background.radius
                    }
                }
            }
        }
    }

    // !!!!!!!!!! Делегат товара !!!!!!!!!!
    component TmcDelegate: GroupBox {
        Style.class: "pickListDelegate"
        padding: 12
        contentItem: ColumnLayout {
            spacing: 0
            Image {
                Layout.fillWidth: true
                Layout.preferredHeight: 100
                source: image
                fillMode: Image.PreserveAspectFit
            }
            Item { Layout.preferredHeight: 10 }
            BasicLabel {
                Style.class: "pickListDelegateName"
                Layout.fillWidth: true
                Layout.preferredHeight: 54
                text: name
                elide: Label.ElideRight
            }
            Item { Layout.preferredHeight: 8 }
            BasicLabel {
                Style.class: "pickListDelegateQuantity"
                Layout.fillWidth: true
                text: quant ?? ""
            }
            Item { Layout.preferredHeight: 8 }
            BasicLabel {
                Style.class: "pickListDelegatePrice"
                Layout.fillWidth: true
                horizontalAlignment: Label.AlignRight
                text: Utils.formatMoneyItem(price)
            }
        }
    }

    // !!!!!!!!!! Контент !!!!!!!!!!
    contentItem: RowLayout {
        spacing: 24
        ColumnLayout {
            spacing: 0
            // Путь до текущего каталога
            ListView {
                Layout.fillWidth: true
                Layout.preferredHeight: 24
                spacing: 4
                orientation: ListView.Horizontal
                model: ListModel { id: headerModel }
                enabled: false
                delegate: RowLayout {
                    spacing: 4
                    BasicLabel {
                        Style.class: "pickListHeader20px400"
                        text: "/"
                        visible: index !== 0
                    }
                    BasicLabel {
                        Style.class: index === 0 ? "pickListHeader20px700" : "pickListHeader20px400"
                        text: name
                    }
                }
            }
            Item { Layout.preferredHeight: 16 }
            // Поле ввода
            BasicTextField {
                id: inputField
                Layout.fillWidth: true
                Layout.preferredHeight: 64
                Style.class: "pickListTextField"
                text: params.presetFilter
                leftPadding: 12
                rightPadding: 12
                BasicLabel {
                    Style.class: "documentOpenInputPlaceholder"
                    anchors.left: parent.left
                    anchors.bottom: parent.bottom
                    anchors.margins: 12
                    visible: inputField.text.length === 0
                    Tr { tr: params.filterMessage }
                }
                Image {
                    anchors.right: parent.right
                    anchors.rightMargin: 12
                    anchors.verticalCenter: parent.verticalCenter
                    width: 30
                    height: 30
                    source: Style.parameter("icon.textFieldClear")
                    MouseArea { anchors.fill: parent; onClicked: inputField.clear() }
                }
                // Делей изменения фильтра
                Timer {
                    id: inputTimer
                    interval: 1000
                    onTriggered: mainModel.filter = inputField.text
                }
                onTextChanged: {
                    if (text.length === 0) {
                        inputTimer.stop()
                        mainModel.filter = text
                    }
                    if (text.length < 4) {
                        inputTimer.stop()
                        return
                    }
                    inputTimer.restart()
                }
            }
            Item { Layout.preferredHeight: 12 }
            // Пиклист
            SwipeMenu {
                id: menu
                Layout.fillWidth: true
                Layout.fillHeight: true
                model: PickListModel { id: mainModel; groupId: rootGroup }
                rows: 2
                columns: 5
                spacing: 12
                delegate: DelegateChooser {
                    role: "type"
                    DelegateChoice {
                        roleValue: PickListModel.GroupType
                        delegate: GroupDelegate {
                            x: menu.cellX(index)
                            y: menu.cellY(index)
                            width: menu.cellWidth
                            height: menu.cellHeight
                            onClicked: push(name, identifier)
                        }
                    }
                    DelegateChoice {
                        roleValue: PickListModel.TmcType
                        delegate: TmcDelegate {
                            x: menu.cellX(index)
                            y: menu.cellY(index)
                            width: menu.cellWidth
                            height: menu.cellHeight
                            onClicked: handler.sendAnswer(identifier)
                        }
                    }
                }
            }
            // Кнопки управления
            Item {
                Layout.fillWidth: true
                Layout.preferredHeight: 68
                RowLayout {
                    spacing: 0
                    anchors.right: parent.right
                    BasicButton {
                        Layout.preferredWidth: 112
                        Layout.preferredHeight: 68
                        Style.class: "pickListBack"
                        Tr { code: "pickListBackButton"; defaultMessage: "Назад" }
                        onClicked: {
                            if (inputField.text.length > 0) {
                                inputField.text = ""
                                return
                            }
                            if (stack.length === 0) {
                                handler.sendAnswer()
                                return
                            }
                            pop()
                        }
                    }
                    readonly property bool additionalNavigation: Style.parameter("capacitiveTouchscreen") === "false"
                    Item { Layout.preferredWidth: 32; visible: parent.additionalNavigation }
                    BasicButton {
                        Style.class: "pickListLeft"
                        Layout.preferredWidth: 68
                        Layout.preferredHeight: 68
                        icon: Style.parameter("icon.pickListArrowLeft")
                        enabled: menu.pageIndex > 0
                        visible: parent.additionalNavigation
                        onClicked: menu.pageIndex--
                    }
                    Item { Layout.preferredWidth: 16; visible: parent.additionalNavigation }
                    BasicButton {
                        Style.class: "pickListRight"
                        Layout.preferredWidth: 68
                        Layout.preferredHeight: 68
                        icon: Style.parameter("icon.pickListArrowRight")
                        enabled: menu.pageIndex < menu.pageCount - 1
                        visible: parent.additionalNavigation
                        onClicked: menu.pageIndex++
                    }
                }
            }
        }
        // Правое меню с корневыми каталогами
        ListView {
            Layout.fillHeight: true
            Layout.preferredWidth: 124
            Layout.topMargin: 40
            clip: true
            spacing: 8
            model: PickListModel { id: sideModel; groupId: rootGroup }
            delegate: BasicButton {
                Style.class: "pickListSideMenu"
                width: parent.width
                height: 68
                text: name
                textLayout { fillWidth: true; fillHeight: true }
                verticalAlignment: Label.AlignBottom
                onClicked: {
                    stack = [{ name: "", id: root.rootGroup }]
                    mainModel.groupId = identifier
                    currentName = name
                }
            }
        }
    }

    // !!!!!!!!!! Клавиатура !!!!!!!!!!
    Popup {
        id: keyboard
        parent: Overlay.overlay
        x: 0
        y: root.height
        width: root.width
        height: root.width * 0.402
        padding: 32
        closePolicy: Popup.NoAutoClose
        contentItem: InlineKeyboard {
            layoutType: "main"
            onHide: inputField.focus = false
            onEnter: inputField.focus = false
        }
        background: Rectangle {
            color: Style.parameter("color.modalBackground")
            // Тень
            Rectangle {
                anchors.bottom: parent.top
                width: parent.width
                height: 5
                gradient: Gradient {
                    orientation: Gradient.Vertical
                    GradientStop { position: 0.0; color: "transparent" }
                    GradientStop { position: 1.0; color: Style.parameter("color.shadow") }
                }
                opacity: 0.3
            }
        }
        Connections {
            target: inputField
            function onFocusChanged() {
                if (inputField.focus)
                    keyboard.open()
                else
                    keyboard.close()
            }
        }
        enter: Transition { SmoothedAnimation { velocity: 1500; property: "y"; to: root.height - keyboard.height } }
        exit: Transition { SmoothedAnimation { velocity: 1500; property: "y"; to: root.height } }
    }

    footer: BottomPanel {}

    DialogHandler { id: handler }

    Component.onCompleted: {
        headerModel.append({"name": headerText})
        mainModel.update()
    }

    // Перейти к каталогу с кодом id
    function push(name, id) {
        stack.push({ name: root.currentName, id: mainModel.groupId })
        root.currentName = name
        mainModel.groupId = id
    }

    // Вернуться вверх по дереву каталогов
    function pop() {
        let item = stack.pop()
        currentName = item.name
        mainModel.groupId = item.id
    }
}
